﻿using UnityEngine;
using System.Collections;

public class CharacterMove : MonoBehaviour
{
	public float spriteDuration = 0.1f;
	public Sprite[] spritesLeft;
	public Sprite[] spritesRight;
	public Sprite[] spritesUp;
	public Sprite[] spritesDown;
	public Sprite[] spritesIdle;

	private float movingTimeDuration;
	private float movingTimeLeft;
	private Vector3 movingStartPos;
	private Vector3 movingEndPos;
	private Sprite[] movingSprites;

	private Transform _tr;
	private SpriteRenderer _sprite;

	void Start ()
	{
		_tr = transform;
		_sprite = GetComponent<SpriteRenderer>();
	}
	
	void Update ()
	{
		UpdateMovement ();
	}

	public bool IsMoving() { return movingTimeLeft > 0.0f; }


	void UpdateSpriteAnimFrame(Sprite[] sprites, float dur)
	{
		if (sprites != null && sprites.Length > 0)
		{
			int idx = (int)(Mathf.Repeat (Time.timeSinceLevelLoad, dur * sprites.Length) / dur);
			idx = Mathf.Clamp (idx, 0, sprites.Length-1);
			_sprite.sprite = sprites[idx];
		}
	}

	void UpdateMovement()
	{
		if (movingTimeLeft > 0.0f)
		{
			// lerp movement
			float alpha = movingTimeLeft / movingTimeDuration;
			_tr.position = Vector3.Lerp (movingStartPos, movingEndPos, 1.0f-alpha);
			movingTimeLeft -= Time.deltaTime;

			// update sprite
			UpdateSpriteAnimFrame(movingSprites, spriteDuration);
		}
		else
		{
			// not moving, snap to integer positions
			movingTimeLeft = 0.0f;
			Vector3 pos = _tr.position;
			pos.x = Mathf.Round(pos.x);
			pos.y = Mathf.Round(pos.y);
			_tr.position = pos;

			// set idle sprite
			UpdateSpriteAnimFrame(spritesIdle, spriteDuration * 2f);
		}
	}

	public bool Move(float dx, float dy, float duration)
	{
		if (IsMoving())
			return true;

		// check if can walk there
		Vector3 pos = _tr.position;
		int destX = Mathf.RoundToInt(pos.x + dx);
		int destY = Mathf.RoundToInt(pos.y + dy);
		if (!World.instance.IsWalkable(destX, destY))
			return false;

		movingStartPos = pos;
		movingEndPos = pos + new Vector3(dx, dy, 0);
		movingTimeLeft = duration;
		movingTimeDuration = duration;

		if (dx < 0.0f)
			movingSprites = spritesLeft;
		else if (dx > 0.0f)
			movingSprites = spritesRight;
		else if (dy < 0.0f)
			movingSprites = spritesDown;
		else if (dy > 0.0f)
			movingSprites = spritesUp;
		else
			movingSprites = null;

		return true;
	}
}
