﻿using UnityEngine;
using System.Collections;


public class Quests : MonoBehaviour
{
	private static Quests s_Instance;
	public static Quests instance { get { return s_Instance; } }

	public UnityEngine.UI.Image uiBackground;
	public UnityEngine.UI.Text uiText;
	public UnityEngine.UI.Button uiButton;
	public UnityEngine.UI.Image uiHero;
	public UnityEngine.UI.Image uiArrow;

	public Quest[] predefined;
	public Quest[] randomQuests;

	public AudioClip questSound;

	private int m_PredefinedIndex = -1;
	private int m_RandomIndex = -1;
	private Quest m_ActiveQuest;

	void Awake()
	{
		if (s_Instance != null)
			Debug.LogError ("Quests instance already exists!");
		s_Instance = this;
	}

	void Start ()
	{
		// disable UI
		uiBackground.gameObject.SetActive(false);

		//uiText.enabled = false;
		//for (var i = 0; i < 100; ++i)
		//	Debug.Log (WordGenerator.GetWord());

		NextQuest();
	}

	public void NextQuest()
	{
		var player = PlayerController.instance;
		if (!player)
			return;

		int iter = 0;
		Quest q;
		if (m_PredefinedIndex < predefined.Length-1)
		{
			// next predefined quest
			++m_PredefinedIndex;
			q = predefined[m_PredefinedIndex];
		}
		else
		{
			// random quest (different from previous random quest)
			int idx = 0;
			iter = 0;
			do {
				idx = Random.Range (0, randomQuests.Length);
				++iter;
			} while (idx == m_RandomIndex && iter < 10);
			m_RandomIndex = idx;
			q = randomQuests[m_RandomIndex];
		}

		// pick location for quest
		iter = 0;
		var qx = 0;
		var qy = 0;
		var plx = player.transform.position.x;
		var ply = player.transform.position.y;

		// ensure we have world spawned around us
		World.instance.EnsureHasTiles((int)(plx-q.distance*2), (int)(ply-q.distance*2), (int)(plx+q.distance*2), (int)(ply+q.distance*2));

		var dist = q.distance;
		do {
			var angle = Random.value * Mathf.PI * 2.0f;
			var dx = Mathf.Cos (angle) * dist;
			var dy = Mathf.Sin (angle) * dist;
			qx = (int)(plx + dx);
			qy = (int)(ply + dy);
			if (iter % 10 == 0)
				dist *= 1.1f;
			if (iter > 1000)
				break;
			++iter;
		} while(!World.instance.IsWalkable(qx, qy));

		// spawn quest
		m_ActiveQuest = Instantiate(q, new Vector3(qx, qy, 0), Quaternion.identity) as Quest;
	}

	void Update()
	{
		if (m_ActiveQuest && PlayerController.instance)
		{
			var playerPos = PlayerController.instance.transform.position;
			var qPos = m_ActiveQuest.transform.position;
			var angle = Mathf.Atan2 ((qPos-playerPos).y, (qPos-playerPos).x) * Mathf.Rad2Deg - 90.0f;
			var rot = Quaternion.Euler(0,0,angle);
			uiArrow.transform.localRotation = rot;
		}
	}
}
